@@ -55,8 +55,9 @@ const struct option cf_options[] = {
 	{"background",      no_argument,       NULL, 'b'},
 	{"script",          required_argument, NULL, 'c'},
 	{"debug",           no_argument,       NULL, 'd'},
-	{"reconfigure",     no_argument,       NULL, 'e'},
+	{"env",             required_argument, NULL, 'e'},
 	{"config",          required_argument, NULL, 'f'},
+	{"reconfigure",     no_argument,       NULL, 'g'},
 	{"hostname",        optional_argument, NULL, 'h'},
 	{"vendorclassid",   optional_argument, NULL, 'i'},
 	{"release",         no_argument,       NULL, 'k'},
@@ -319,7 +320,8 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 	struct rt *rt;
 
 	switch(opt) {
-	case 'e': /* FALLTHROUGH */
+	case 'f': /* FALLTHROUGH */
+	case 'g': /* FALLTHROUGH */
 	case 'n': /* FALLTHROUGH */
 	case 'x': /* FALLTHROUGH */
 	case 'T': /* We need to handle non interface options */
@@ -333,6 +335,9 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 	case 'd':
 		ifo->options |= DHCPCD_DEBUG;
 		break;
+	case 'e':
+		add_environ(ifo, arg, 1);
+		break;
 	case 'h':
 		if (arg) {
 			s = parse_string(ifo->hostname,
@@ -750,8 +755,11 @@ read_config(const char *file,
 
 	/* Parse our options file */
 	f = fopen(file ? file : CONFIG, "r");
-	if (!f)
+	if (f == NULL) {
+		if (file != NULL)
+			syslog(LOG_ERR, "fopen `%s': %m", file);
 		return ifo;
+	}
 
 	while ((line = get_line(f))) {
 		option = strsep(&line, " \t");
